/*
 * @license
 * Copyright 2020-2020, Red Huang
 * Released under the MIT License
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _classCallCheck = _interopDefault(require('@babel/runtime/helpers/classCallCheck'));
var _createClass = _interopDefault(require('@babel/runtime/helpers/createClass'));

var vLazyRefDirective = {
  inserted: function inserted(el, _ref, vnode) {
    var value = _ref.value;
    var context = vnode.context;

    if (!context._lazyRefsInfo) {
      return;
    }

    if (vnode.componentInstance) {
      context._lazyRefsInfo.runResolver(value, vnode.componentInstance);
    } else {
      context._lazyRefsInfo.runResolver(value, el);
    }

    return;
  },
  unbind: function unbind(el, _ref2, vnode) {
    var value = _ref2.value;
    var context = vnode.context;

    if (!context._lazyRefsInfo) {
      return;
    }

    context._lazyRefsInfo.removeElCache(value, el);

    return;
  }
};

var LazyRefsInfo = /*#__PURE__*/function () {
  function LazyRefsInfo() {
    _classCallCheck(this, LazyRefsInfo);

    this._lazyRefs = Object.create(null);
    this._resolvers = Object.create(null);
    this._elCache = Object.create(null);
  }

  _createClass(LazyRefsInfo, [{
    key: "getResolvers",
    value: function getResolvers(refName) {
      return this._resolvers[refName];
    }
  }, {
    key: "removeResolver",
    value: function removeResolver(refName) {
      if (this._resolvers[refName]) {
        delete this._resolvers[refName];
      }
    }
  }, {
    key: "removeElCache",
    value: function removeElCache(refName) {
      if (this._elCache) {
        delete this._elCache[refName];
      }
    }
  }, {
    key: "runResolver",
    value: function runResolver(refName, el) {
      var resolver = this.getResolvers(refName);

      if (resolver) {
        resolver(el);
        this.removeResolver(refName);
      }

      this._elCache[refName] = el;
    }
  }, {
    key: "getLazyRefs",
    value: function getLazyRefs(refName) {
      var _this = this;

      if (this._elCache[refName]) {
        return Promise.resolve(this._elCache[refName]);
      }

      if (!this._lazyRefs[refName]) {
        var defer = new Promise(function (resolve, reject) {
          _this._resolvers[refName] = resolve;
        });
        this._lazyRefs[refName] = defer;
      }

      return this._lazyRefs[refName];
    }
  }]);

  return LazyRefsInfo;
}();

var vLazyRefMixin = {
  created: function created() {
    this._lazyRefsInfo = new LazyRefsInfo();
  },
  beforeDestroy: function beforeDestroy() {
    this._lazyRefsInfo = null;
  },
  methods: {
    '$getLazyRefs': function $getLazyRefs(refName) {
      return this._lazyRefsInfo.getLazyRefs(refName);
    }
  }
};

var vLazyRefPlugin = {
  install: function install(Vue) {
    Vue.directive('lazy-ref', vLazyRefDirective);
    Vue.mixin(vLazyRefMixin);
  }
};

exports.default = vLazyRefPlugin;
exports.vLazyRefPlugin = vLazyRefPlugin;
